@@ -6,12 +6,15 @@
 #'@keywords internal
 #'@return Returns a vector consisting of the number
 #'of levels preceeding each parameter (including the intercept)
-constructRunMatrix = function(rowIndices, candidateList) {
+constructRunMatrix = function(rowIndices, candidateList, augment=NULL) {
   trials = length(rowIndices)
-  runMatrix = as.data.frame(candidateList[1:trials,])
-  colnames(runMatrix) = colnames(candidateList)
+  run_matrix = as.data.frame(candidateList[1:trials,])
+  colnames(run_matrix) = colnames(candidateList)
   for(i in 1:trials) {
-    runMatrix[i,] = candidateList[rowIndices[i],]
+    run_matrix[i,] = candidateList[rowIndices[i],]
   }
-  return(runMatrix)
+  if(!is.null(augment)) {
+    run_matrix[1:nrow(augment),] = augment
+  }
+  return(run_matrix)
 }
